@@ -1,12 +1,10 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2010-2013 Raphaël Barrois
+# Copyright (c) 2010-2020 Raphaël Barrois
 # This software is distributed under the two-clause BSD license.
 
 
 """Stacking filesystems."""
 
-from __future__ import absolute_import, unicode_literals
-
 import collections
 import contextlib
 import dbm
@@ -34,7 +32,7 @@ class ChrootFS(base.WrappingFS):
     All paths are converted on the fly.
     """
     def __init__(self, external_root=ROOT, internal_root=ROOT, **kwargs):
-        super(ChrootFS, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.external_root = external_root
         self.internal_root = internal_root
 
@@ -64,10 +62,10 @@ class ReadOnlyFS(base.WrappingFS):
     # Read
     # ----
 
-    def _access(self, path, mode):
+    def _access(self, path, mode, follow=True):
         if mode & os.W_OK:
             return False
-        return self.wrapped.access(path, mode)
+        return self.wrapped.access(path, mode, follow=follow)
 
     # def _listdir: unchanged
     # def _lstat: unchanged
@@ -119,7 +117,7 @@ class ReadOnlyFS(base.WrappingFS):
 # ============
 
 
-class BaseWhiteoutCache(object):
+class BaseWhiteoutCache:
     def __contains__(self, key):
         raise NotImplementedError()
 
@@ -183,7 +181,7 @@ class WhiteoutFS(base.WrappingFS):
     )
 
     def __init__(self, whiteout_cache, **kwargs):
-        super(WhiteoutFS, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.whiteout_cache = whiteout_cache
 
     def __del__(self):
@@ -222,7 +220,7 @@ class WhiteoutFS(base.WrappingFS):
     # Read
     # ----
 
-    def _access(self, path, mode):
+    def _access(self, path, mode, follow=True):
         try:
             self._check_path(path)
         except OSError as e:
@@ -231,7 +229,7 @@ class WhiteoutFS(base.WrappingFS):
             else:
                 raise
 
-        return self.wrapped.access(path, mode)
+        return self.wrapped.access(path, mode, follow=follow)
 
     def _listdir(self, path):
         with self._manage_whiteout(path, for_creation=False):
@@ -328,7 +326,7 @@ class UnionFS(base.BaseFS):
     )
 
     def __init__(self, strict=False, **kwargs):
-        super(UnionFS, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.strict = strict
         self._branches = {}
         self._sorted_branches = []
@@ -341,7 +339,7 @@ class UnionFS(base.BaseFS):
     def has_feature(self, feature):
         if feature == self.FEATURE_READONLY:
             return not self._write_branches
-        return super(UnionFS, self).has_feature(feature)
+        return super().has_feature(feature)
 
     # Branches management
     # -------------------
@@ -477,8 +475,7 @@ class UnionFS(base.BaseFS):
                 branch.fs.mkdir(component)
                 self._copy_stat(component, branch, old_stat)
 
-    def _copy_object(self, path, target_branch, old_branch,
-            for_overwrite=False):
+    def _copy_object(self, path, target_branch, old_branch, for_overwrite=False):
 
         old_stat = old_branch.fs.lstat(path)
         if stat.S_ISDIR(old_stat.st_mode):
@@ -507,8 +504,7 @@ class UnionFS(base.BaseFS):
     _EXIST_YES = 'yes'
     _EXIST_NO = 'no'
 
-    def _copy_on_write(self, target_path, branch,
-            expected=_EXIST_ANY, for_overwrite=False):
+    def _copy_on_write(self, target_path, branch, expected=_EXIST_ANY, for_overwrite=False):
         # 1. Ensure the parent dir is valid somewhere
         parent = os.path.dirname(target_path)
         if not self.isdir(parent):
@@ -548,7 +544,7 @@ class UnionFS(base.BaseFS):
     # Read
     # ----
 
-    def _access(self, path, mode):
+    def _access(self, path, mode, follow=True):
         try:
             branch, _stats = self._get_read_branch(path)
         except OSError as e:
@@ -556,7 +552,7 @@ class UnionFS(base.BaseFS):
                 return False
             # Worse!
             raise
-        return branch.fs.access(path, mode)
+        return branch.fs.access(path, mode, follow=follow)
 
     def _get_dir_branches(self, path):
         """Fetch the list of branches where a directory exists."""
@@ -671,8 +667,7 @@ class UnionFS(base.BaseFS):
         return branch.fs.rmdir(path)
 
     def _unlink(self, path):
-        branch = self._get_write_branch(path,
-            expected=self._EXIST_YES, for_overwrite=True)
+        branch = self._get_write_branch(path, expected=self._EXIST_YES, for_overwrite=True)
         return branch.fs.unlink(path)
 
 
@@ -693,7 +688,7 @@ def _has_access(mode, uid, gid, target_mode, target_uid, target_gid):
     return False
 
 
-class FakeFSObject(object):
+class FakeFSObject:
     BASE_ST_MOD = 0
 
     def __init__(self, path, mode, uid, gid):
@@ -761,7 +756,7 @@ class FakeFSObject(object):
     is_symlink = False
 
 
-class BufferWrapper(object):
+class BufferWrapper:
     def __init__(self, buf):
         self._buf = buf
 
@@ -784,7 +779,7 @@ class FakeFile(FakeFSObject):
     is_file = True
 
     def __init__(self, **kwargs):
-        super(FakeFile, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.content = io.BytesIO()
 
     def open_binary(self, mode):
@@ -798,7 +793,6 @@ class FakeFile(FakeFSObject):
         return io.TextIOWrapper(BufferWrapper(self.content), encoding=encoding)
 
 
-
 class FakeDir(FakeFSObject):
     """A fake directory.
 
@@ -809,7 +803,7 @@ class FakeDir(FakeFSObject):
     is_dir = True
 
     def __init__(self, **kwargs):
-        super(FakeDir, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.contents = {}
 
     def __contains__(self, path):
@@ -859,11 +853,7 @@ class FakeDir(FakeFSObject):
             raise exceptions.EACCES(full_path)
         if self.mode & stat.S_ISGID:
             gid = self.gid
-        new_link = FakeSymlink(target,
-            mode=mode,
-            uid=uid,
-            gid=gid,
-        )
+        new_link = FakeSymlink(target, mode=mode, uid=uid, gid=gid)
         self.contents[relative_path] = new_link
         return new_link
 
@@ -900,7 +890,7 @@ class FakeSymlink(FakeFSObject):
     is_symlink = True
 
     def __init__(self, target, **kwargs):
-        super(FakeSymlink, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.target = target
 
     def stat(self):
@@ -912,7 +902,7 @@ class FakeSymlink(FakeFSObject):
 
 class MemoryFS(base.BaseFS):
     def __init__(self, *args, **kwargs):
-        super(MemoryFS, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.fake_root = FakeDir(
             path=ROOT,
             mode=self.default_dir_mode,
@@ -948,9 +938,9 @@ class MemoryFS(base.BaseFS):
     # Read
     # ----
 
-    def _access(self, path, mode):
+    def _access(self, path, mode, follow=True):
         try:
-            target = self._get(path)
+            target = self._get(path, follow_symlinks=follow)
         except KeyError:
             return False
         return target.access(mode)
@@ -983,7 +973,8 @@ class MemoryFS(base.BaseFS):
             if helpers.is_readonly_open_mode(mode):
                 raise exceptions.ENOENT(path)
             parent = self._get_parent(path)
-            target = parent.make_file(os.path.basename(path),
+            target = parent.make_file(
+                os.path.basename(path),
                 mode=self.default_file_mode,
                 uid=self.default_uid,
                 gid=self.default_gid,
@@ -1018,7 +1009,8 @@ class MemoryFS(base.BaseFS):
         if path in parent:
             raise exceptions.EEXIST(link_name)
 
-        new_link = parent.make_symlink(path, target,
+        new_link = parent.make_symlink(
+            path, target,
             mode=self.default_symlink_mode,
             uid=self.default_uid,
             gid=self.default_gid,
@@ -1028,7 +1020,8 @@ class MemoryFS(base.BaseFS):
 
     def _mkdir(self, path):
         parent = self._get_parent(path)
-        new_dir = parent.make_subdir(os.path.basename(path),
+        new_dir = parent.make_subdir(
+            os.path.basename(path),
             mode=self.default_dir_mode,
             uid=self.default_uid,
             gid=self.default_gid,
@@ -1071,7 +1064,7 @@ class MountFS(base.BaseFS):
         '/bar/baz' on that file-system.
     """
     def __init__(self, **kwargs):
-        super(MountFS, self).__init__(**kwargs)
+        super().__init__(**kwargs)
         self.filesystems = {}
         self._sorted_filesystems = []
 
@@ -1154,9 +1147,9 @@ class MountFS(base.BaseFS):
     # Read
     # ----
 
-    def _access(self, path, mode):
+    def _access(self, path, mode, follow=True):
         relpath, subfs = self._map_path(path)
-        return subfs.access(relpath, mode)
+        return subfs.access(relpath, mode, follow=follow)
 
     def _listdir(self, path):
         relpath, subfs = self._map_path(path)
@@ -1204,8 +1197,10 @@ class MountFS(base.BaseFS):
         relative_link, link_subfs = self._map_path(link_name)
         relative_target, target_subfs = self._map_path(target)
         if target_subfs != link_subfs:
-            raise exceptions.FSError("Can't create cross-filesystem symlink from %r to %r."
-                    % (link_subfs, target_subfs))
+            raise exceptions.FSError(
+                "Can't create cross-filesystem symlink from %r to %r."
+                % (link_subfs, target_subfs)
+            )
         return link_subfs.symlink(relative_link, relative_target)
 
     # Delete
